import json
from magnet.datastructs import jsonPos
import copy

class magnetParser:
    def parseJson(self, jsonStr):
        obj = json.loads(jsonStr)
        corporas = []
        pos = jsonPos()
        pos.add(dict, 'blocks')
        for i in range(len(obj['blocks'])):
            b = obj['blocks'][i]
            np = copy.deepcopy(pos)
            np.add(list, i)
            self.parseElement(b, corporas, np)
        return corporas


    def parseElement(self, eleObj, corporas, pos: jsonPos):
        if type(eleObj) != dict:
            if type(eleObj) == list:
                for i in range(len(eleObj)):
                    l = eleObj[i]
                    np = copy.deepcopy(pos)
                    np.add(list, i)
                    self.parseElement(l, corporas, np)
            else:
                corporas.append((eleObj, pos))
        else:
            if 'c' not in eleObj.keys():
                return
            if 't' in eleObj.keys():
                if eleObj['t'] == 'Image':
                    return
            if type(eleObj['c']) == list:
                pos.add(dict, 'c')
                for i in range(len(eleObj['c'])):
                    c = eleObj['c'][i]
                    np = copy.deepcopy(pos)
                    np.add(list, i)
                    self.parseElement(c, corporas, np)
            else:
                np = copy.deepcopy(pos)
                np.add(dict, 'c')
                self.parseElement(eleObj['c'], corporas, np)



import pypandoc
from magnet.snowflake import magnetSnowflake
from magnet.parser import magnetParser
from magnet.replacer import magnetReplacer


def hello():
    return "Hello Magnet"


class magnetEngine:
    def __init__(self):
        self.tempFolder = 'temps'
        self._Snowflake = magnetSnowflake(1, 2)
        self._Parser = magnetParser()

    def shutdown(self):
        pass

    def importFile(self, filePath):
        taskID = self._Snowflake.next()
        taskPath = self.tempFolder + '/' + str(taskID) + '.json'
        pypandoc.convert(
            filePath,
            'json',
            format='docx',
            outputfile=taskPath,
            extra_args=[
                '--extract-media=temps/'+str(taskID)
            ])
        return taskID

    def getCorporas(self, taskID):
        taskPath = self.tempFolder + '/' + str(taskID) + '.json'
        with open(taskPath, encoding='utf-8') as OJson:
            return self._Parser.parseJson(OJson.read())

    def setParallelCorporas(self, taskID, ParallelCorporas):
        corporas = self.getCorporas(taskID)
        if type(ParallelCorporas) != list or \
                len(ParallelCorporas) != len(corporas):
            raise Exception()
        taskPath = self.tempFolder + '/' + str(taskID) + '.json'
        jsonStr = ''
        with open(taskPath, encoding='utf-8') as OJson:
            jsonStr = magnetReplacer().replaceJson(OJson.read(), corporas, ParallelCorporas)
        with open(taskPath, 'w', encoding='utf-8') as OJson:
            OJson.write(jsonStr)

    def exportFile(self, filePath, taskID):
        taskPath = self.tempFolder + '/' + str(taskID) + '.json'
        pypandoc.convert(
            taskPath,
            'docx',
            format='json',
            outputfile=filePath)


def createEngine():
    return magnetEngine()



import json


class magnetReplacer:
    def replaceJson(self, jsonStr, corporas, parallelCorporas):
        jsonObj = json.loads(jsonStr)
        for i in range(len(corporas)):
            self.replaceElement(
                corporas[i][1].Pos, parallelCorporas[i], jsonObj)
        return json.dumps(jsonObj)

    def replaceElement(self, index: list, replacement, obj):
        if len(index) == 1:
            if type(replacement) == type(obj[index[0][1]]):
                obj[index[0][1]] = replacement
        else:
            self.replaceElement(index[1:], replacement, obj[index[0][1]])



import time
import logging


class magnetSnowflake:
    def __init__(self, worker_id, data_center_id):
        self.log = logging.getLogger(__name__)

        self.twepoch = 1142974214000

        self.worker_id_bits = 5
        self.data_center_id_bits = 5
        self.max_worker_id = -1 ^ (-1 << self.worker_id_bits)
        self.max_data_center_id = -1 ^ (-1 << self.data_center_id_bits)
        self.sequence_bits = 12
        self.worker_id_shift = self.sequence_bits
        self.data_center_id_shift = self.sequence_bits + self.worker_id_bits
        self.timestamp_left_shift = self.sequence_bits + \
            self.worker_id_bits + self.data_center_id_bits
        self.sequence_mask = -1 ^ (-1 << self.sequence_bits)

        self._snowflakeGenerator = self._generator(1, 1)

    def snowflake_to_timestamp(self, _id):
        _id = _id >> 22
        _id += self.twepoch
        _id = _id / 1000
        return _id

    def _generator(self, worker_id, data_center_id, sleep=lambda x: time.sleep(x/1000.0)):
        assert worker_id >= 0 and worker_id <= self.max_worker_id
        assert data_center_id >= 0 and data_center_id <= self.max_data_center_id

        last_timestamp = -1
        sequence = 0

        while True:
            timestamp = int(time.time()*1000)

            if last_timestamp > timestamp:
                log.warning(
                    "clock is moving backwards. waiting until %i" % last_timestamp)
                sleep(last_timestamp-timestamp)
                continue

            if last_timestamp == timestamp:
                sequence = (sequence + 1) & self.sequence_mask
                if sequence == 0:
                    log.warning("sequence overrun")
                    sequence = -1 & self.sequence_mask
                    sleep(1)
                    continue
            else:
                sequence = 0

            last_timestamp = timestamp

            yield (
                ((timestamp-self.twepoch) << self.timestamp_left_shift) |
                (data_center_id << self.data_center_id_shift) |
                (worker_id << self.worker_id_shift) |
                sequence)

    def next(self):
        return next(self._snowflakeGenerator)



import setuptools


with open("README.md", "r", encoding='UTF-8') as RMF:
    long_description = RMF.read()

setuptools.setup(
    name='magnet4c',
    version='0.0.1',
    author='huang825172',
    author_email='1048035187@qq.com',
    description="Corpora operator for multiple file formats.",
    keywords='corpora parallel file extract replace',
    long_description=long_description,
    url = 'http://at.senhacore.xyz:3000/huang825172/Magnet',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License',
    ],
    python_requires='>=3.6',
    install_requires=[
        'pypandoc',
    ],
    package_dir={
        '':'src',
    },
    packages=setuptools.find_packages(where='src')
)


class jsonPos:
    def __init__(self):
        self.Pos = []

    def add(self, type, key):
        if type not in [list, dict]:
            raise Exception()
        self.Pos.append((type, key))

    def __repr__(self):
        rstr = ''
        for p in self.Pos:
            if p[0] == list:
                rstr += '['
                rstr += str(p[1])
            else:
                rstr += '{'
                rstr += str(p[1])
        return rstr
